%rotina que inicializa os pesos - arranjo unidimensional em anel
%parmetros: X - matriz dos dados de entrada - NxM

%Esta inicializao visa minimizar a violao topolgica na construo do
%mapa, ou seja, esperamos que aps a auto-organizao do mapa, neurnios
%vizinhos no mapa tenham vetores de pesos prximos no espao dos dados

function W = inicializa_pesos(X, neuronios)

%N = nmero de componentes e M = nmero de dados (padres) de entrada
[N,M] = size(X);
%ponto mdio dos dados - centro do anel
medio = mean(X,2)'; 
%limitantes dos padres de entrada
limite_max=max(max(X));
limite_min=min(min(X));
%foramos os dados de entrada terem mdia zero
Xmz = X - repmat(medio',1,M);
%matriz de autocorrelao - PCA
R = Xmz*Xmz';
%autovalores e autovetores
[V,~] = eig(R);
%dois maiores autovalores definem as duas componentes principais
direcao = [V(:,N-1) V(:,N)];
%raio do anel
raio = abs(0.2*(limite_max - limite_min));
%ngulo de variao
angles = linspace(0,2*pi,neuronios);
%montamos a matriz dos pesos dos neurnios - um anel no espao dos dados
W = cos(angles)'*raio*direcao(:,1)'+sin(angles)'*raio*direcao(:,2)'+repmat(medio',1,neuronios)';
W = W'; %manter coerncia entre as rotinas
