%treinamento de um mapa de Kohonen - arranjo unidimensional - anel
%mecanismos de poda e insero automtica de neurnios
%matriz X - colunas representam os padres de entrada(N atributos x M padres)

function [W,Index,neuronios] = kohonen(X,neuronios,gama_inicial,radius_inicial,LIMIAR,LIMITE_TAXA,PERIODO,MAX_ITERATION)

%N = nmero de componentes e M = nmero de dados (padres) de entrada
[~,M] = size(X);

%nmero de iteraes
iteration = 1;
%taxa de aprendizado
gama = gama_inicial; 
%radius definir a vizinhana - no incio, ambos vizinhos so afetados
%conforme o nmero de iteraes aumenta, no teremos mais o ajuste
radius = radius_inicial;

%matriz com o ndice dos vizinhos topolgicos - anel
vetor = (1:neuronios)';
Index = [circshift(vetor,-1) circshift(vetor,1)];
%vetor que contabiliza as vitrias dos neurnios
wins = zeros(1,neuronios);

%inicializao da matriz de pesos
W = inicializa_pesos(X,neuronios);
%exibe configurao inicial - pesos e dados
figure(1); plot_SOM(X,W,neuronios);
title('Configurao inicial dos neurnios(o)');

%contador do nmero de neurnios prximos aos padres
cont = 0; 

%critrio de parada:
%   nmero mximo de pocas
%   contador do nmero de neurnios que se encontram muito prximos aos
%   dados - ou seja, cuja dist. ao dado (cidade) mais prximo  inferior ao
%   LIMIAR
%   taxa de aprendizado maior que um valor mnimo permitido

while iteration < MAX_ITERATION && cont < M && gama > LIMITE_TAXA
    
    %a cada PERIODO iteraes, zeramos o contador de vitrias
    if(mod(iteration,PERIODO)==0)
        wins = zeros(1,neuronios);
    end
    
    %ordena aleatoriamente os padres de entrada
    X = X(:,randperm(M));
    %contador do nmero de neurnios prximos aos padres
    cont = 0;
   
    %apresenta os M padroes
    for i=1:1:M
        %determinamos o neurnio vencedor
        [indice, value] = vencedor(X(:,i),W);
        %contabiliza uma vitria para o neurnio "indice"
        wins(indice) = wins(indice) + 1;
        if value < LIMIAR
           %se todos os neurnios esto com proximidade menor que um LIMIAR
           cont = cont + 1;
        end
        %ajustamos os pesos do vencedor e dos neurnios vizinhos
        W = ajuste_peso(W, X, Index, indice, gama, i, radius);
    end
    
    %ajusta a taxa de aprendizado gama e a vizinhana
    gama = gama_inicial*exp(-(iteration)/MAX_ITERATION);  %const de tempo
    radius = radius_inicial*exp(-(iteration)/MAX_ITERATION);
    
    %a cada PERIODO iteraes, fazemos as operaes de poda e insero de neurnios
    if(mod(iteration,PERIODO)==0)
        %etapa de poda - neurnios que nunca vencem
        [W,wins,neuronios,Index] = poda(W,wins,neuronios,Index);
        %etapa de insero - prximo a neurnios que vencem muito
        [W,wins,neuronios,Index] = insere(W,wins,neuronios,Index);
    end
    %exibio dos parmetros
    fprintf('Iteracao:%d \t Taxa (Gama):%1.4f \t Raio:%d \t N:%d\n',iteration,gama,round(radius),neuronios);
    figure(2); plot_SOM(X,W,neuronios); title(['Configurao dos neurnios - iterao ' num2str(iteration)]); drawnow;
    iteration = iteration + 1;
end